@@ -2,17 +2,15 @@
 --
 -- See @{02-arrays.md.Useful_Operations_on_Tables|the Guide}
 --
--- Dependencies: `pl.utils`
+-- Dependencies: `pl.utils`, `pl.types`
 -- @module pl.tablex
 local utils = require ('pl.utils')
+local types = require ('pl.types')
 local getmetatable,setmetatable,require = getmetatable,setmetatable,require
 local tsort,append,remove = table.sort,table.insert,table.remove
-local min,max = math.min,math.max
-local pairs,type,unpack,next,select,tostring = pairs,type,unpack,next,select,tostring
+local min = math.min
+local pairs,type,unpack,select,tostring = pairs,type,utils.unpack,select,tostring
 local function_arg = utils.function_arg
-local Set = utils.stdmt.Set
-local List = utils.stdmt.List
-local Map = utils.stdmt.Map
 local assert_arg = utils.assert_arg
 
 local tablex = {}
@@ -20,18 +18,17 @@ local tablex = {}
 -- generally, functions that make copies of tables try to preserve the metatable.
 -- However, when the source has no obvious type, then we attach appropriate metatables
 -- like List, Map, etc to the result.
-local function setmeta (res,tbl,def)
-    local mt = getmetatable(tbl) or def
-    return setmetatable(res, mt)
+local function setmeta (res,tbl,pl_class)
+    local mt = getmetatable(tbl) or pl_class and require('pl.' .. pl_class)
+    return mt and setmetatable(res, mt) or res
 end
 
-local function makelist (res)
-    return setmetatable(res,List)
+local function makelist(l)
+    return setmetatable(l, require('pl.List'))
 end
 
-local function check_meta (val)
-    if type(val) == 'table' then return true end
-    return getmetatable(val)
+local function makemap(m)
+    return setmetatable(m, require('pl.Map'))
 end
 
 local function complain (idx,msg)
@@ -39,33 +36,27 @@ local function complain (idx,msg)
 end
 
 local function assert_arg_indexable (idx,val)
-    local mt = check_meta(val)
-    if mt == true then return end
-    if not(mt and mt.__len and mt.__index) then
+    if not types.is_indexable(val) then
         complain(idx,"indexable")
     end
 end
 
 local function assert_arg_iterable (idx,val)
-    local mt = check_meta(val)
-    if mt == true then return end
-    if not(mt and mt.__pairs) then
+    if not types.is_iterable(val) then
         complain(idx,"iterable")
     end
 end
 
 local function assert_arg_writeable (idx,val)
-    local mt = check_meta(val)
-    if mt == true then return end
-    if not(mt and mt.__newindex) then
+    if not types.is_writeable(val) then
         complain(idx,"writeable")
     end
 end
 
-
 --- copy a table into another, in-place.
--- @param t1 destination table
--- @param t2 source (any iterable object)
+-- @within Copying
+-- @tab t1 destination table
+-- @tab t2 source (actually any iterable object)
 -- @return first table
 function tablex.update (t1,t2)
     assert_arg_writeable(1,t1)
@@ -82,7 +73,7 @@ end
 -- be greater or equal. The difference gives the size of
 -- the hash part, for practical purposes. Works for any
 -- object with a __pairs metamethod.
--- @param t a table
+-- @tab t a table
 -- @return the size
 function tablex.size (t)
     assert_arg_iterable(1,t)
@@ -92,7 +83,8 @@ function tablex.size (t)
 end
 
 --- make a shallow copy of a table
--- @param t an iterable source
+-- @within Copying
+-- @tab t an iterable source
 -- @return new table
 function tablex.copy (t)
     assert_arg_iterable(1,t)
@@ -105,7 +97,8 @@ end
 
 --- make a deep copy of a table, recursively copying all the keys and fields.
 -- This will also set the copied table's metatable to that of the original.
---  @param t A table
+-- @within Copying
+-- @tab t A table
 -- @return new table
 function tablex.deepcopy(t)
     if type(t) ~= 'table' then return t end
@@ -126,10 +119,11 @@ local abs, deepcompare = math.abs
 
 --- compare two values.
 -- if they are tables, then compare their keys and fields recursively.
+-- @within Comparing
 -- @param t1 A value
 -- @param t2 A value
--- @param ignore_mt if true, ignore __eq metamethod (default false)
--- @param eps if defined, then used for any number comparisons
+-- @bool[opt] ignore_mt if true, ignore __eq metamethod (default false)
+-- @number[opt] eps if defined, then used for any number comparisons
 -- @return true or false
 function tablex.deepcompare(t1,t2,ignore_mt,eps)
     local ty1 = type(t1)
@@ -160,9 +154,10 @@ end
 deepcompare = tablex.deepcompare
 
 --- compare two arrays using a predicate.
--- @param t1 an array
--- @param t2 an array
--- @param cmp A comparison function
+-- @within Comparing
+-- @array t1 an array
+-- @array t2 an array
+-- @func cmp A comparison function
 function tablex.compare (t1,t2,cmp)
     assert_arg_indexable(1,t1)
     assert_arg_indexable(2,t2)
@@ -175,8 +170,9 @@ function tablex.compare (t1,t2,cmp)
 end
 
 --- compare two list-like tables using an optional predicate, without regard for element order.
--- @param t1 a list-like table
--- @param t2 a list-like table
+-- @within Comparing
+-- @array t1 a list-like table
+-- @array t2 a list-like table
 -- @param cmp A comparison function (may be nil)
 function tablex.compare_no_order (t1,t2,cmp)
     assert_arg_indexable(1,t1)
@@ -205,9 +201,10 @@ end
 --- return the index of a value in a list.
 -- Like string.find, there is an optional index to start searching,
 -- which can be negative.
--- @param t A list-like table (i.e. with numerical indices)
+-- @within Finding
+-- @array t A list-like table
 -- @param val A value
--- @param idx index to start; -1 means last element,etc (default 1)
+-- @int idx index to start; -1 means last element,etc (default 1)
 -- @return index of value or nil if not found
 -- @usage find({10,20,30},20) == 2
 -- @usage find({'a','b','a','c'},'a',2) == 3
@@ -224,7 +221,8 @@ end
 --- return the index of a value in a list, searching from the end.
 -- Like string.find, there is an optional index to start searching,
 -- which can be negative.
--- @param t A list-like table (i.e. with numerical indices)
+-- @within Finding
+-- @array t A list-like table
 -- @param val A value
 -- @param idx index to start; -1 means last element,etc (default 1)
 -- @return index of value or nil if not found
@@ -241,8 +239,9 @@ end
 
 
 --- return the index (or key) of a value in a table using a comparison function.
--- @param t A table
--- @param cmp A comparison function
+-- @within Finding
+-- @tab t A table
+-- @func cmp A comparison function
 -- @param arg an optional second argument to the function
 -- @return index of value, or nil if not found
 -- @return value returned by comparison function
@@ -257,8 +256,8 @@ function tablex.find_if(t,cmp,arg)
 end
 
 --- return a list of all values in a table indexed by another list.
--- @param tbl a table
--- @param idx an index table (a list of keys)
+-- @tab tbl a table
+-- @array idx an index table (a list of keys)
 -- @return a list-like table
 -- @usage index_by({10,20,30,40},{2,4}) == {20,40}
 -- @usage index_by({one=1,two=2,three=3},{'one','three'}) == {1,3}
@@ -269,14 +268,15 @@ function tablex.index_by(tbl,idx)
     for i = 1,#idx do
         res[i] = tbl[idx[i]]
     end
-    return setmeta(res,tbl,List)
+    return setmeta(res,tbl,'List')
 end
 
 --- apply a function to all values of a table.
 -- This returns a table of the results.
 -- Any extra arguments are passed to the function.
--- @param fun A function that takes at least one argument
--- @param t A table
+-- @within MappingAndFiltering
+-- @func fun A function that takes at least one argument
+-- @tab t A table
 -- @param ... optional arguments
 -- @usage map(function(v) return v*v end, {10,20,30,fred=2}) is {100,400,900,fred=4}
 function tablex.map(fun,t,...)
@@ -292,8 +292,9 @@ end
 --- apply a function to all values of a list.
 -- This returns a table of the results.
 -- Any extra arguments are passed to the function.
--- @param fun A function that takes at least one argument
--- @param t a table (applies to array part)
+-- @within MappingAndFiltering
+-- @func fun A function that takes at least one argument
+-- @array t a table (applies to array part)
 -- @param ... optional arguments
 -- @return a list-like table
 -- @usage imap(function(v) return v*v end, {10,20,30,fred=2}) is {100,400,900}
@@ -304,12 +305,13 @@ function tablex.imap(fun,t,...)
     for i = 1,#t do
         res[i] = fun(t[i],...) or false
     end
-    return setmeta(res,t,List)
+    return setmeta(res,t,'List')
 end
 
 --- apply a named method to values from a table.
--- @param name the method name
--- @param t a list-like table
+-- @within MappingAndFiltering
+-- @string name the method name
+-- @array t a list-like table
 -- @param ... any extra arguments to the method
 function tablex.map_named_method (name,t,...)
     utils.assert_string(1,name)
@@ -320,14 +322,13 @@ function tablex.map_named_method (name,t,...)
         local fun = val[name]
         res[i] = fun(val,...)
     end
-    return setmeta(res,t,List)
+    return setmeta(res,t,'List')
 end
 
-
 --- apply a function to all values of a table, in-place.
 -- Any extra arguments are passed to the function.
--- @param fun A function that takes at least one argument
--- @param t a table
+-- @func fun A function that takes at least one argument
+-- @tab t a table
 -- @param ... extra arguments
 function tablex.transform (fun,t,...)
     assert_arg_iterable(1,t)
@@ -337,27 +338,31 @@ function tablex.transform (fun,t,...)
     end
 end
 
---- generate a table of all numbers in a range
--- @param start  number
--- @param finish number
--- @param step optional increment (default 1 for increasing, -1 for decreasing)
+--- generate a table of all numbers in a range.
+-- This is consistent with a numerical for loop.
+-- @int start  number
+-- @int finish number
+-- @int[opt=1] step  make this negative for start < finish
 function tablex.range (start,finish,step)
-    if start == finish then return {start}
-    elseif start > finish then return {}
+    local res
+    step = step or 1
+    if start == finish then
+        res = {start}
+    elseif (start > finish and step > 0) or (finish > start and step < 0) then
+        res = {}
+    else
+        local k = 1
+        res = {}
+        for i=start,finish,step do res[k]=i; k=k+1 end
     end
-    local res = {}
-    local k = 1
-    if not step then
-        if finish > start then step = finish > start and 1 or -1 end
-    end
-    for i=start,finish,step do res[k]=i; k=k+1 end
-    return res
+    return makelist(res)
 end
 
 --- apply a function to values from two tables.
--- @param fun a function of at least two arguments
--- @param t1 a table
--- @param t2 a table
+-- @within MappingAndFiltering
+-- @func fun a function of at least two arguments
+-- @tab t1 a table
+-- @tab t2 a table
 -- @param ... extra arguments
 -- @return a table
 -- @usage map2('+',{1,2,3,m=4},{10,20,30,m=40}) is {11,22,23,m=44}
@@ -369,14 +374,15 @@ function tablex.map2 (fun,t1,t2,...)
     for k,v in pairs(t1) do
         res[k] = fun(v,t2[k],...)
     end
-    return setmeta(res,t1,List)
+    return setmeta(res,t1,'List')
 end
 
 --- apply a function to values from two arrays.
 -- The result will be the length of the shortest array.
--- @param fun a function of at least two arguments
--- @param t1 a list-like table
--- @param t2 a list-like table
+-- @within MappingAndFiltering
+-- @func fun a function of at least two arguments
+-- @array t1 a list-like table
+-- @array t2 a list-like table
 -- @param ... extra arguments
 -- @usage imap2('+',{1,2,3,m=4},{10,20,30,m=40}) is {11,22,23}
 function tablex.imap2 (fun,t1,t2,...)
@@ -391,15 +397,19 @@ function tablex.imap2 (fun,t1,t2,...)
 end
 
 --- 'reduce' a list using a binary function.
--- @param fun a function of two arguments
--- @param t a list-like table
+-- @func fun a function of two arguments
+-- @array t a list-like table
+-- @array memo optional initial memo value. Defaults to first value in table.
 -- @return the result of the function
 -- @usage reduce('+',{1,2,3,4}) == 10
-function tablex.reduce (fun,t)
+function tablex.reduce (fun,t,memo)
     assert_arg_indexable(2,t)
     fun = function_arg(1,fun)
     local n = #t
-    local res = t[1]
+    if n == 0 then
+        return memo
+    end
+    local res = memo and fun(memo, t[1]) or t[1]
     for i = 2,n do
         res = fun(res,t[i])
     end
@@ -408,10 +418,11 @@ end
 
 --- apply a function to all elements of a table.
 -- The arguments to the function will be the value,
--- the key and <i>finally</i> any extra arguments passed to this function.
--- Note that the Lua 5.0 function table.foreach passed the <i>key</i> first.
--- @param t a table
--- @param fun a function with at least one argument
+-- the key and _finally_ any extra arguments passed to this function.
+-- Note that the Lua 5.0 function table.foreach passed the _key_ first.
+-- @within Iterating
+-- @tab t a table
+-- @func fun a function with at least one argument
 -- @param ... extra arguments
 function tablex.foreach(t,fun,...)
     assert_arg_iterable(1,t)
@@ -423,9 +434,10 @@ end
 
 --- apply a function to all elements of a list-like table in order.
 -- The arguments to the function will be the value,
--- the index and <i>finally</i> any extra arguments passed to this function
--- @param t a table
--- @param fun a function with at least one argument
+-- the index and _finally_ any extra arguments passed to this function
+-- @within Iterating
+-- @array t a table
+-- @func fun a function with at least one argument
 -- @param ... optional arguments
 function tablex.foreachi(t,fun,...)
     assert_arg_indexable(1,t)
@@ -435,13 +447,13 @@ function tablex.foreachi(t,fun,...)
     end
 end
 
-
 --- Apply a function to a number of tables.
 -- A more general version of map
 -- The result is a table containing the result of applying that function to the
 -- ith value of each table. Length of output list is the minimum length of all the lists
--- @param fun a function of n arguments
--- @param ... n tables
+-- @within MappingAndFiltering
+-- @func fun a function of n arguments
+-- @tab ... n tables
 -- @usage mapn(function(x,y,z) return x+y+z end, {1,2,3},{10,20,30},{100,200,300}) is {111,222,333}
 -- @usage mapn(math.max, {1,20,300},{10,2,3},{100,200,100}) is	{100,200,300}
 -- @param fun A function that takes as many arguments as there are tables
@@ -466,10 +478,11 @@ end
 
 --- call the function with the key and value pairs from a table.
 -- The function can return a value and a key (note the order!). If both
--- are not nil, then this pair is inserted into the result. If only value is not nil, then
--- it is appended to the result.
--- @param fun A function which will be passed each key and value as arguments, plus any extra arguments to pairmap.
--- @param t A table
+-- are not nil, then this pair is inserted into the result: if the key already exists, we convert the value for that
+-- key into a table and append into it. If only value is not nil, then it is appended to the result.
+-- @within MappingAndFiltering
+-- @func fun A function which will be passed each key and value as arguments, plus any extra arguments to pairmap.
+-- @tab t A table
 -- @param ... optional arguments
 -- @usage pairmap(function(k,v) return v end,{fred=10,bonzo=20}) is {10,20} _or_ {20,10}
 -- @usage pairmap(function(k,v) return {k,v},k end,{one=1,two=2}) is {one={'one',1},two={'two',2}}
@@ -480,7 +493,15 @@ function tablex.pairmap(fun,t,...)
     for k,v in pairs(t) do
         local rv,rk = fun(k,v,...)
         if rk then
-            res[rk] = rv
+			if res[rk] then
+				if type(res[rk]) == 'table' then
+					table.insert(res[rk],rv)
+				else
+					res[rk] = {res[rk], rv}
+				end
+			else
+            	res[rk] = rv
+			end
         else
             res[#res+1] = rv
         end
@@ -491,7 +512,8 @@ end
 local function keys_op(i,v) return i end
 
 --- return all the keys of a table in arbitrary order.
---  @param t A table
+-- @within Extraction
+--  @tab t A table
 function tablex.keys(t)
     assert_arg_iterable(1,t)
     return makelist(tablex.pairmap(keys_op,t))
@@ -500,7 +522,8 @@ end
 local function values_op(i,v) return v end
 
 --- return all the values of the table in arbitrary order
---  @param t A table
+-- @within Extraction
+--  @tab t A table
 function tablex.values(t)
     assert_arg_iterable(1,t)
     return makelist(tablex.pairmap(values_op,t))
@@ -510,31 +533,31 @@ local function index_map_op (i,v) return i,v end
 
 --- create an index map from a list-like table. The original values become keys,
 -- and the associated values are the indices into the original list.
--- @param t a list-like table
+-- @array t a list-like table
 -- @return a map-like table
 function tablex.index_map (t)
     assert_arg_indexable(1,t)
-    return setmetatable(tablex.pairmap(index_map_op,t),Map)
+    return makemap(tablex.pairmap(index_map_op,t))
 end
 
 local function set_op(i,v) return true,v end
 
 --- create a set from a list-like table. A set is a table where the original values
 -- become keys, and the associated values are all true.
--- @param t a list-like table
+-- @array t a list-like table
 -- @return a set (a map-like table)
 function tablex.makeset (t)
     assert_arg_indexable(1,t)
-    return setmetatable(tablex.pairmap(set_op,t),Set)
+    return setmetatable(tablex.pairmap(set_op,t),require('pl.Set'))
 end
 
-
 --- combine two tables, either as union or intersection. Corresponds to
 -- set operations for sets () but more general. Not particularly
 -- useful for list-like tables.
--- @param t1 a table
--- @param t2 a table
--- @param dup true for a union, false for an intersection.
+-- @within Merging
+-- @tab t1 a table
+-- @tab t2 a table
+-- @bool dup true for a union, false for an intersection.
 -- @usage merge({alice=23,fred=34},{bob=25,fred=34}) is {fred=34}
 -- @usage merge({alice=23,fred=34},{bob=25,fred=34},true) is {bob=25,fred=34,alice=23}
 -- @see tablex.index_map
@@ -550,40 +573,60 @@ function tablex.merge (t1,t2,dup)
         res[k] = v
       end
     end
-    return setmeta(res,t1,Map)
+    return setmeta(res,t1,'Map')
+end
+
+--- the union of two map-like tables.
+-- If there are duplicate keys, the second table wins.
+-- @tab t1 a table
+-- @tab t2 a table
+-- @treturn tab
+-- @see tablex.merge
+function tablex.union(t1, t2)
+    return tablex.merge(t1, t2, true)
+end
+
+--- the intersection of two map-like tables.
+-- @tab t1 a table
+-- @tab t2 a table
+-- @treturn tab
+-- @see tablex.merge
+function tablex.intersection(t1, t2)
+    return tablex.merge(t1, t2, false)
 end
 
 --- a new table which is the difference of two tables.
 -- With sets (where the values are all true) this is set difference and
 -- symmetric difference depending on the third parameter.
--- @param s1 a map-like table or set
--- @param s2 a map-like table or set
--- @param symm symmetric difference (default false)
+-- @within Merging
+-- @tab s1 a map-like table or set
+-- @tab s2 a map-like table or set
+-- @bool symm symmetric difference (default false)
 -- @return a map-like table or set
 function tablex.difference (s1,s2,symm)
     assert_arg_iterable(1,s1)
     assert_arg_iterable(2,s2)
     local res = {}
     for k,v in pairs(s1) do
-        if not s2[k] then res[k] = v end
+        if s2[k] == nil then res[k] = v end
     end
     if symm then
         for k,v in pairs(s2) do
-            if not s1[k] then res[k] = v end
+            if s1[k] == nil then res[k] = v end
         end
     end
-    return setmeta(res,s1,Map)
+    return setmeta(res,s1,'Map')
 end
 
 --- A table where the key/values are the values and value counts of the table.
--- @param t a list-like table
--- @param cmp a function that defines equality (otherwise uses ==)
+-- @array t a list-like table
+-- @func cmp a function that defines equality (otherwise uses ==)
 -- @return a map-like table
 -- @see seq.count_map
 function tablex.count_map (t,cmp)
     assert_arg_indexable(1,t)
     local res,mask = {},{}
-    cmp = function_arg(2,cmp)
+    cmp = function_arg(2,cmp or '==')
     local n = #t
     for i = 1,#t do
         local v = t[i]
@@ -593,19 +636,21 @@ function tablex.count_map (t,cmp)
             res[v] = 1  -- there's at least one instance
             for j = i+1,n do
                 local w = t[j]
-                if cmp and cmp(v,w) or v == w then
+                local ok = cmp(v,w)
+                if ok then
                     res[v] = res[v] + 1
                     mask[w] = true
                 end
             end
         end
     end
-    return setmetatable(res,Map)
+    return makemap(res)
 end
 
---- filter a table's values using a predicate function
--- @param t a list-like table
--- @param pred a boolean function
+--- filter an array's values using a predicate function
+-- @within MappingAndFiltering
+-- @array t a list-like table
+-- @func pred a boolean function
 -- @param arg optional argument to be passed as second argument of the predicate
 function tablex.filter (t,pred,arg)
     assert_arg_indexable(1,t)
@@ -618,12 +663,14 @@ function tablex.filter (t,pred,arg)
             k = k + 1
         end
     end
-    return setmeta(res,t,List)
+    return setmeta(res,t,'List')
 end
 
 --- return a table where each element is a table of the ith values of an arbitrary
 -- number of tables. It is equivalent to a matrix transpose.
+-- @within Merging
 -- @usage zip({10,20,30},{100,200,300}) is {{10,100},{20,200},{30,300}}
+-- @array ... arrays to be zipped
 function tablex.zip(...)
     return tablex.mapn(function(...) return {...} end,...)
 end
@@ -655,24 +702,26 @@ function _copy (dest,src,idest,isrc,nsrc,clean_tail)
     return dest
 end
 
---- copy an array into another one, resizing the destination if necessary. <br>
--- @param dest a list-like table
--- @param src a list-like table
--- @param idest where to start copying values from source (default 1)
--- @param isrc where to start copying values into destination (default 1)
--- @param nsrc number of elements to copy from source (default source size)
+--- copy an array into another one, clearing `dest` after `idest+nsrc`, if necessary.
+-- @within Copying
+-- @array dest a list-like table
+-- @array src a list-like table
+-- @int[opt=1] idest where to start copying values into destination
+-- @int[opt=1] isrc where to start copying values from source
+-- @int[opt=#src] nsrc number of elements to copy from source
 function tablex.icopy (dest,src,idest,isrc,nsrc)
     assert_arg_indexable(1,dest)
     assert_arg_indexable(2,src)
     return _copy(dest,src,idest,isrc,nsrc,true)
 end
 
---- copy an array into another one. <br>
--- @param dest a list-like table
--- @param src a list-like table
--- @param idest where to start copying values from source (default 1)
--- @param isrc where to start copying values into destination (default 1)
--- @param nsrc number of elements to copy from source (default source size)
+--- copy an array into another one.
+-- @within Copying
+-- @array dest a list-like table
+-- @array src a list-like table
+-- @int[opt=1] idest where to start copying values into destination
+-- @int[opt=1] isrc where to start copying values from source
+-- @int[opt=#src] nsrc number of elements to copy from source
 function tablex.move (dest,src,idest,isrc,nsrc)
     assert_arg_indexable(1,dest)
     assert_arg_indexable(2,src)
@@ -693,28 +742,29 @@ end
 -- If first or last are negative then they are relative to the end of the list
 -- eg. sub(t,-2) gives last 2 entries in a list, and
 -- sub(t,-4,-2) gives from -4th to -2nd
--- @param t a list-like table
--- @param first An index
--- @param last An index
+-- @within Extraction
+-- @array t a list-like table
+-- @int first An index
+-- @int last An index
 -- @return a new List
 function tablex.sub(t,first,last)
     assert_arg_indexable(1,t)
     first,last = tablex._normalize_slice(t,first,last)
     local res={}
     for i=first,last do append(res,t[i]) end
-    return setmeta(res,t,List)
+    return setmeta(res,t,'List')
 end
 
 --- set an array range to a value. If it's a function we use the result
 -- of applying it to the indices.
--- @param t a list-like table
+-- @array t a list-like table
 -- @param val a value
--- @param i1 start range (default 1)
--- @param i2 end range (default table size)
+-- @int[opt=1] i1 start range
+-- @int[opt=#t] i2 end range
 function tablex.set (t,val,i1,i2)
     assert_arg_indexable(1,t)
     i1,i2 = i1 or 1,i2 or #t
-    if utils.is_callable(val) then
+    if types.is_callable(val) then
         for i = i1,i2 do
             t[i] = val(i)
         end
@@ -726,8 +776,8 @@ function tablex.set (t,val,i1,i2)
 end
 
 --- create a new array of specified size with initial value.
--- @param n size
--- @param val initial value (can be nil, but don't expect # to work!)
+-- @int n size
+-- @param val initial value (can be `nil`, but don't expect `#` to work!)
 -- @return the table
 function tablex.new (n,val)
     local res = {}
@@ -736,17 +786,20 @@ function tablex.new (n,val)
 end
 
 --- clear out the contents of a table.
--- @param t a table
+-- @array t a list
 -- @param istart optional start position
 function tablex.clear(t,istart)
     istart = istart or 1
     for i = istart,#t do remove(t) end
 end
 
---- insert values into a table. <br>
--- insertvalues(t, [pos,] values) <br>
--- similar to table.insert but inserts values from given table "values",
--- not the object itself, into table "t" at position "pos".
+--- insert values into a table.
+-- similar to `table.insert` but inserts values from given table `values`,
+-- not the object itself, into table `t` at position `pos`.
+-- @within Copying
+-- @array t the list
+-- @int[opt] position (default is at end)
+-- @array values
 function tablex.insertvalues(t, ...)
     assert_arg(1,t,'table')
     local pos, values
@@ -768,9 +821,10 @@ function tablex.insertvalues(t, ...)
 end
 
 --- remove a range of values from a table.
--- @param t a list-like table
--- @param i1 start index
--- @param i2 end index
+-- End of range may be negative.
+-- @array t a list-like table
+-- @int i1 start index
+-- @int i2 end index
 -- @return the table
 function tablex.removevalues (t,i1,i2)
     assert_arg(1,t,'table')
@@ -803,9 +857,10 @@ _find = function (t,value,tables)
 end
 
 --- find a value in a table by recursive search.
--- @param t the table
+-- @within Finding
+-- @tab t the table
 -- @param value the value
--- @param exclude any tables to avoid searching
+-- @array[opt] exclude any tables to avoid searching
 -- @usage search(_G,math.sin,{package.path}) == 'math.sin'
 -- @return a fieldspec, e.g. 'a.b' or 'math.sin'
 function tablex.search (t,value,exclude)
@@ -818,40 +873,44 @@ function tablex.search (t,value,exclude)
 end
 
 --- return an iterator to a table sorted by its keys
--- @param t the table
--- @param f an optional comparison function (f(x,y) is true if x < y)
+-- @within Iterating
+-- @tab t the table
+-- @func f an optional comparison function (f(x,y) is true if x < y)
 -- @usage for k,v in tablex.sort(t) do print(k,v) end
 -- @return an iterator to traverse elements sorted by the keys
 function tablex.sort(t,f)
-   local keys = {}
-   for k in pairs(t) do keys[#keys + 1] = k end
-   tsort(keys,f)
-   local i = 0
-   return function()
-      i = i + 1
-      return keys[i], t[keys[i]]
-   end
+    local keys = {}
+    for k in pairs(t) do keys[#keys + 1] = k end
+    tsort(keys,f)
+    local i = 0
+    return function()
+        i = i + 1
+        return keys[i], t[keys[i]]
+    end
 end
 
 --- return an iterator to a table sorted by its values
--- @param t the table
--- @param f an optional comparison function (f(x,y) is true if x < y)
+-- @within Iterating
+-- @tab t the table
+-- @func f an optional comparison function (f(x,y) is true if x < y)
 -- @usage for k,v in tablex.sortv(t) do print(k,v) end
 -- @return an iterator to traverse elements sorted by the values
 function tablex.sortv(t,f)
-   local rev = {}
-   for k,v in pairs(t) do rev[v] = k end
-   local next = tablex.sort(rev,f)
-   return function()
-      local value,key = next()
-      return key,value
-   end
+    f = function_arg(2, f or '<')
+    local keys = {}
+    for k in pairs(t) do keys[#keys + 1] = k end
+    tsort(keys,function(x, y) return f(t[x], t[y]) end)
+    local i = 0
+    return function()
+        i = i + 1
+        return keys[i], t[keys[i]]
+    end
 end
 
 --- modifies a table to be read only.
 -- This only offers weak protection. Tables can still be modified with
--- table.insert and rawset.
--- @param t the table
+-- `table.insert` and `rawset`.
+-- @tab t the table
 -- @return the table read only.
 function tablex.readonly(t)
     local mt = {
@@ -865,6 +924,4 @@ function tablex.readonly(t)
     return setmetatable({}, mt)
 end
 
-
-
 return tablex
